@@ -40,7 +40,6 @@
 #define N_(String) (String)
 
 /* map configuration: */
-#define USE_CAIRO
 #define ENABLE_OSD
 #undef OSD_BALLOON
 #define OSD_SCALE
@@ -57,31 +56,32 @@
 #include <hildon/hildon-defines.h>
 /* only maemo devices up to version 4 have a fullscreen button */
 #if (MAEMO_VERSION_MAJOR < 5)
-#define OSM_GPS_MAP_KEY_FULLSCREEN  HILDON_HARDKEY_FULLSCREEN
+#define MAP_KEY_FULLSCREEN  HILDON_HARDKEY_FULLSCREEN
 #else
-#define OSM_GPS_MAP_KEY_FULLSCREEN  'f'
+#define MAP_KEY_FULLSCREEN  'f'
 #endif
-#define OSM_GPS_MAP_KEY_ZOOMIN      HILDON_HARDKEY_INCREASE
-#define OSM_GPS_MAP_KEY_ZOOMOUT     HILDON_HARDKEY_DECREASE
+#define MAP_KEY_ZOOMIN      HILDON_HARDKEY_INCREASE
+#define MAP_KEY_ZOOMOUT     HILDON_HARDKEY_DECREASE
 #else
-#define OSM_GPS_MAP_KEY_FULLSCREEN  GDK_F11
-#define OSM_GPS_MAP_KEY_ZOOMIN      '+'
-#define OSM_GPS_MAP_KEY_ZOOMOUT     '-'
+#define MAP_KEY_FULLSCREEN  GDK_F11
+#define MAP_KEY_ZOOMIN      '+'
+#define MAP_KEY_ZOOMOUT     '-'
 #endif
 
-#define OSM_GPS_MAP_KEY_UP          GDK_Up
-#define OSM_GPS_MAP_KEY_DOWN        GDK_Down
-#define OSM_GPS_MAP_KEY_LEFT        GDK_Left
-#define OSM_GPS_MAP_KEY_RIGHT       GDK_Right
+#define MAP_KEY_UP          GDK_Up
+#define MAP_KEY_DOWN        GDK_Down
+#define MAP_KEY_LEFT        GDK_Left
+#define MAP_KEY_RIGHT       GDK_Right
 
 /* specify OSD colors explicitely. Otherwise gtk default */
 /* colors are used. fremantle always uses gtk defaults */
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+#ifndef MAEMO5
 #define OSD_COLOR_BG         1, 1, 1         // white background
 #define OSD_COLOR            0.5, 0.5, 1     // light blue border and controls
 #define OSD_COLOR_DISABLED   0.8, 0.8, 0.8   // light grey disabled controls
 #define OSD_SHADOW_ENABLE
 #else
+/* fremantle has controls at botton (for fringer friendlyness) */
 #define OSD_Y  -10
 #endif
 
@@ -89,5 +89,6 @@
 #define OSD_GPS_BUTTON       // display a GPS button
 #define OSD_NO_DPAD          // no direction arrows (map is panned)
 #define OSD_SOURCE_SEL       // display source selection tab
+#define OSD_BALLOON
 
 #endif // CONFIG_H
