@@ -24,12 +24,26 @@
 #include "config.h"
 #include "menu.h"
 #include "track.h"
+#include "geonames.h"
 
 #ifdef MAEMO5
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 #endif
 
+#include <locale.h>
+
+/* any defined key enables key support */
+#if (defined(MAP_KEY_FULLSCREEN) || \
+     defined(MAP_KEY_ZOOMIN) || \
+     defined(MAP_KEY_ZOOMOUT) || \
+     defined(MAP_KEY_UP) || \
+     defined(MAP_KEY_DOWN) || \
+     defined(MAP_KEY_LEFT) || \
+     defined(MAP_KEY_RIGHT))
+#include <gdk/gdkkeysyms.h>
+#endif
+
 #include <locale.h>
 #include <libintl.h>
 
@@ -243,17 +257,17 @@ static GtkWidget *map_new(void) {
   /* ~/.osm-gps-map on standard PC     (users home) */
   /* /home/user/MyDocs/.maps on Maemo5 (vfat on internal card) */
   /* /media/mmc2/osm-gps-map on Maemo4 (vfat on internal card) */
+  char *path;
+
 #if !defined(USE_MAEMO)
   char *p = getenv("HOME");
   if(!p) p = "/tmp"; 
-  char *path = g_strdup_printf("%s/.osm-gps-map", p);
+  path = g_strdup_printf("%s/.osm-gps-map", p);
 #else
 #if MAEMO_VERSION_MAJOR == 5
   /* early maep releases used the ext3 for the tile cache */
 #define OLD_PATH "/home/user/.osm-gps-map"
 #define NEW_PATH "/home/user/MyDocs/.maps"
-  char *path = NULL;
-
   /* check if the old path exists, and is not a symlink, then use it */
   if( g_file_test(OLD_PATH, G_FILE_TEST_IS_DIR) &&
      !g_file_test(OLD_PATH, G_FILE_TEST_IS_SYMLINK))
@@ -261,18 +275,14 @@ static GtkWidget *map_new(void) {
   else
     path = g_strdup(NEW_PATH);
 #else
-  char *path = g_strdup("/media/mmc2/osm-gps-map");
+  path = g_strdup("/media/mmc2/osm-gps-map");
 #endif
 #endif
-
-  printf("Storing tile cache at %s\n", path);
+  GConfClient *gconf_client = gconf_client_get_default();
+  gint source = gconf_get_int(gconf_client, GCONF_KEY_SOURCE, MAP_SOURCE);
 
   const char *proxy = get_proxy_uri();
 
-  GConfClient *gconf_client = gconf_client_get_default();
-  gint source = gconf_get_int(gconf_client, 
-			      GCONF_KEY_SOURCE, MAP_SOURCE);
-
   /* get zoom, latitude and longitude from gconf if possible */
   gint zoom = gconf_get_int(gconf_client, GCONF_KEY_ZOOM, 3);
   gfloat lat = gconf_get_float(gconf_client, GCONF_KEY_LATITUDE, 50.0);
@@ -280,15 +290,41 @@ static GtkWidget *map_new(void) {
     
   GtkWidget *widget = g_object_new(OSM_TYPE_GPS_MAP,
 		 "map-source",               source,
-                 "tile-cache",               path,
+		 "tile-cache",               OSM_GPS_MAP_CACHE_FRIENDLY,
+                 "tile-cache-base",          path,
 		 "auto-center",              FALSE,
 		 "record-trip-history",      FALSE, 
 		 "show-trip-history",        FALSE, 
+		 "gps-track-point-radius",   10,
 		 proxy?"proxy-uri":NULL,     proxy,
                  NULL);
 
   g_free(path);
 
+  OsmGpsMap *map = OSM_GPS_MAP(widget);
+
+#ifdef MAP_KEY_FULLSCREEN
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_FULLSCREEN, MAP_KEY_FULLSCREEN);
+#endif
+#ifdef MAP_KEY_ZOOMIN
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_ZOOMIN, MAP_KEY_ZOOMIN);
+#endif
+#ifdef MAP_KEY_ZOOMOUT
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_ZOOMOUT, MAP_KEY_ZOOMOUT);
+#endif
+#ifdef MAP_KEY_UP
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_UP, MAP_KEY_UP);
+#endif
+#ifdef MAP_KEY_DOWN
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_DOWN, MAP_KEY_DOWN);
+#endif
+#ifdef MAP_KEY_LEFT
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_LEFT, MAP_KEY_LEFT);
+#endif
+#ifdef MAP_KEY_RIGHT
+  osm_gps_map_set_keyboard_shortcut(map, OSM_GPS_MAP_KEY_RIGHT, MAP_KEY_RIGHT);
+#endif
+
   osm_gps_map_set_mapcenter(OSM_GPS_MAP(widget), lat, lon, zoom);
 
   osm_gps_map_osd_classic_init(OSM_GPS_MAP(widget));
@@ -348,10 +384,12 @@ static void on_map_destroy (GtkWidget *widget, gpointer data) {
 
 int main(int argc, char *argv[]) {
 
+#if 0
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
+#endif
 
   /* prepare thread system. the map uses soup which in turn seems */
   /* to need this */
@@ -365,7 +403,12 @@ int main(int argc, char *argv[]) {
   g_set_application_name("Mæp");
   
   /* Create HildonWindow and set it to HildonProgram */
+#ifdef MAEMO5
+  GtkWidget *window = hildon_stackable_window_new();
+#else
   GtkWidget *window = hildon_window_new();
+#endif
+
   hildon_program_add_window(program, HILDON_WINDOW(window));
 
   g_object_set_data(G_OBJECT(window), "osso-context",
@@ -406,7 +449,9 @@ int main(int argc, char *argv[]) {
 
   gtk_box_pack_start_defaults(GTK_BOX(vbox), map);
 
+  
   track_restore(map);
+  geonames_wikipedia_restore(map);
 
   gtk_widget_show_all(GTK_WIDGET(window));
   gtk_main();
