@@ -27,16 +27,24 @@
 /* OSD_DIAMETER */
 /* OSD_X, OSD_Y */
 
-#ifndef USE_CAIRO
-#error "OSD control display lacks a non-cairo implementation!"
-#endif
-
 #include <cairo.h>
 
 #include "osm-gps-map.h"
 #include "converter.h"
 #include "osm-gps-map-osd-classic.h"
 
+static OsmGpsMapSource_t map_sources[] = {
+    OSM_GPS_MAP_SOURCE_OPENSTREETMAP,
+    OSM_GPS_MAP_SOURCE_OPENSTREETMAP_RENDERER,
+    OSM_GPS_MAP_SOURCE_OPENCYCLEMAP,
+    OSM_GPS_MAP_SOURCE_OSM_PUBLIC_TRANSPORT,
+    OSM_GPS_MAP_SOURCE_GOOGLE_STREET,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_STREET,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_SATELLITE,
+    OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_HYBRID
+};
+static int num_map_sources = sizeof(map_sources)/sizeof(map_sources[0]);
+
 //the osd controls
 typedef struct {
     /* the offscreen representation of the OSD */
@@ -123,8 +131,8 @@ typedef struct {
 #endif
 
 #define BALLOON_BORDER               (BALLOON_CORNER_RADIUS/2)
-#define BALLOON_WIDTH                (BALLOON_AREA_WIDTH + 2 * BALLOON_BORDER)
-#define BALLOON_HEIGHT               (BALLOON_AREA_HEIGHT + 2 * BALLOON_BORDER)
+#define BALLOON_WIDTH                (priv->balloon.rect.w + 2 * BALLOON_BORDER)
+#define BALLOON_HEIGHT               (priv->balloon.rect.h + 2 * BALLOON_BORDER)
 #define BALLOON_TRANSPARENCY         0.8
 #define POINTER_HEIGHT                20
 #define POINTER_FOOT_WIDTH            20
@@ -135,8 +143,6 @@ typedef struct {
 #define BALLOON_W  (BALLOON_WIDTH + BALLOON_SHADOW)
 #define BALLOON_H  (BALLOON_HEIGHT + POINTER_HEIGHT + BALLOON_SHADOW)
 
-#define CLOSE_BUTTON_RADIUS   (BALLOON_CORNER_RADIUS)
-
 /* draw the bubble shape. this is used twice, once for the shape and once */
 /* for the shadow */
 static void 
@@ -179,7 +185,6 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
 
     if(!priv->balloon.surface)
         return;
-
     /* get zoom */
     gint zoom;
     g_object_get(OSM_GPS_MAP(osd->widget), "zoom", &zoom, NULL);
@@ -232,13 +237,12 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     priv->balloon.orientation = orientation;
 
     /* calculate bottom/right of box */
-    int x1 = x0 + BALLOON_WIDTH, y1 = y0 + BALLOON_HEIGHT;
+    int x1 = x0 + priv->balloon.rect.w + 2*BALLOON_BORDER;
+    int y1 = y0 + priv->balloon.rect.h + 2*BALLOON_BORDER;
 
     /* save balloon screen coordinates for later use */
     priv->balloon.rect.x = x0 + BALLOON_BORDER;
     priv->balloon.rect.y = y0 + BALLOON_BORDER;
-    priv->balloon.rect.w = x1 - x0 - 2*BALLOON_BORDER;
-    priv->balloon.rect.h = y1 - y0 - 2*BALLOON_BORDER;
 
     g_assert(priv->balloon.surface);
     cairo_t *cr = cairo_create(priv->balloon.surface);
@@ -379,18 +383,40 @@ osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude,
 
     osm_gps_map_osd_clear_balloon (map);
 
-    /* allocate balloon surface */
-    priv->balloon.surface = 
-        cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
-                                   BALLOON_W+2, BALLOON_H+2);
-
     priv->balloon.lat = latitude;
     priv->balloon.lon = longitude;
     priv->balloon.cb = cb;
     priv->balloon.data = data;
     priv->balloon.just_created = TRUE;
-
     priv->balloon.orientation = -1;
+    priv->balloon.rect.w = 0;
+    priv->balloon.rect.h = 0;
+
+    /* set default size and call app callback */
+    if(!priv->balloon.rect.w || !priv->balloon.rect.h) {
+        osm_gps_map_balloon_event_t event;
+        
+        priv->balloon.rect.w = BALLOON_AREA_WIDTH;
+        priv->balloon.rect.h = BALLOON_AREA_HEIGHT;
+
+        /* create some temporary surface, so the callback can */
+        /* e.g. determine text extents etc */
+        cairo_surface_t *surface = 
+            cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 10,10);
+
+        event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_SIZE_REQUEST;
+        event.data.draw.rect = &priv->balloon.rect;
+        event.data.draw.cr = cairo_create(surface);
+        priv->balloon.cb(&event, priv->balloon.data);
+
+        cairo_destroy(event.data.draw.cr);
+        cairo_surface_destroy(surface);
+    }
+
+    /* allocate balloon surface */
+    priv->balloon.surface = 
+        cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 
+                                   BALLOON_W+2, BALLOON_H+2);
 
     osd_render_balloon(osd);
 
@@ -690,19 +716,19 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                                     CAIRO_FONT_SLANT_NORMAL,
                                     CAIRO_FONT_WEIGHT_BOLD);
             cairo_set_font_size (cr, OSD_FONT_SIZE);
-            
+
             int i, step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) / 
-                OSM_GPS_MAP_SOURCE_LAST;
-            for(i=OSM_GPS_MAP_SOURCE_NULL+1;i<=OSM_GPS_MAP_SOURCE_LAST;i++) {
+                num_map_sources;
+            for(i=0;i<num_map_sources;i++) {
                 cairo_text_extents_t extents;
-                const char *src = osm_gps_map_source_get_friendly_name(i);
+                const char *src = osm_gps_map_source_get_friendly_name( map_sources[i] );
                 cairo_text_extents (cr, src, &extents);
                 
                 int x = offset + OSD_S_PW + OSD_TEXT_BORDER;
-                int y = offset + step * (i-1) + OSD_TEXT_BORDER;
+                int y = offset + step * i + OSD_TEXT_BORDER;
 
                 /* draw filled rectangle if selected */
-                if(source == i) {
+                if(source == map_sources[i]) {
                     cairo_rectangle(cr, x - OSD_TEXT_BORDER/2, 
                                     y - OSD_TEXT_SKIP, 
                                     priv->source_sel.width - OSD_TEXT_BORDER, 
@@ -722,7 +748,7 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
                 cairo_show_text (cr, src);
 
                 /* restore color */
-                if(source == i) {
+                if(source == map_sources[i]) {
 #ifndef OSD_COLOR
                     GdkColor fg = osd->widget->style->fg[GTK_STATE_NORMAL];
                     gdk_cairo_set_source_color(cr, &fg);
@@ -810,8 +836,8 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
 
         /* calculate menu size */
         int i, max_h = 0, max_w = 0;
-        for(i=OSM_GPS_MAP_SOURCE_NULL+1;i<=OSM_GPS_MAP_SOURCE_LAST;i++) {
-            const char *src = osm_gps_map_source_get_friendly_name(i);
+        for(i=0;i<num_map_sources;i++) {
+            const char *src = osm_gps_map_source_get_friendly_name( map_sources[i] );
             cairo_text_extents (cr, src, &extents);
 
             if(extents.width > max_w) max_w = extents.width;
@@ -820,7 +846,7 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_destroy(cr);
        
         priv->source_sel.width  = max_w + 2*OSD_TEXT_BORDER;
-        priv->source_sel.height = OSM_GPS_MAP_SOURCE_LAST * 
+        priv->source_sel.height = num_map_sources * 
             (max_h + 2*OSD_TEXT_SKIP) + 2*OSD_TEXT_BORDER;
 
         w = OSD_S_EXP_W;
@@ -951,20 +977,19 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
            y < OSD_S_EXP_H) {
             
             int step = (priv->source_sel.height - 2*OSD_TEXT_BORDER) 
-                / OSM_GPS_MAP_SOURCE_LAST;
+                / num_map_sources;
 
             y -= OSD_TEXT_BORDER - OSD_TEXT_SKIP;
             y /= step;
-            y += 1;
 
             if(down) {
                 gint old = 0;
                 g_object_get(osd->widget, "map-source", &old, NULL);
 
-                if(y > OSM_GPS_MAP_SOURCE_NULL &&
-                   y <= OSM_GPS_MAP_SOURCE_LAST &&
-                   old != y) {
-                    g_object_set(osd->widget, "map-source", y, NULL);
+                if(y >= 0 &&
+                   y < num_map_sources &&
+                   old != map_sources[y]) {
+                    g_object_set(osd->widget, "map-source", map_sources[y], NULL);
                     
                     osd_render_source_sel(osd, TRUE);
                     osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
@@ -1233,8 +1258,6 @@ static float get_distance(float lat1, float lon1, float lat2, float lon2) {
   float aob = acos(cos(lat1) * cos(lat2) * cos(lon2 - lon1) +
 		   sin(lat1) * sin(lat2));
 
-  //  return(aob * 3959.0);   /* great circle radius in miles */
-
   return(aob * 6371000.0);     /* great circle radius in meters */
 }
 
@@ -2030,11 +2053,12 @@ osd_free(osm_gps_map_osd_t *osd)
 #endif
 
 #ifdef OSD_BALLOON
-    if (priv->balloon.surface)
+    if (priv->balloon.surface) 
          cairo_surface_destroy(priv->balloon.surface);
 #endif
 
     g_free(priv);
+    osd->priv = NULL;
 }
 
 static gboolean
